# coding: utf-8

import re

from timeit import default_timer

from django.db import connection
from speedinfo import profiler, settings

try:
    from django.urls import resolve, Resolver404  # Django >= 1.10
except ImportError:
    from django.core.urlresolvers import resolve, Resolver404


class ProfilerMiddleware(object):
    """
    Collects request and response statistics and saves profiler data.
    Unified middleware for all Django versions.
    """
    def __init__(self, get_response=None):
        self.get_response = get_response
        self.force_debug_cursor = False
        self.start_time = 0
        self.exclude_urls_re = [re.compile(pattern) for pattern in settings.SPEEDINFO_EXCLUDE_URLS]

    def match_exclude_urls(self, path):
        """Looks for a match requested page url to the exclude urls list.

        :param str path: Path to the requested page, not including the scheme or domain
        :return: True if path matches to any of the exclude urls
        :rtype: bool
        """
        for url_re in self.exclude_urls_re:
            if url_re.match(path):
                return True

        return False

    def get_view_name(self, request):
        """Returns full view name from request, eg. 'app.module.view_name'.

        :param request: Request object
        :type request: :class:`django.http.HttpRequest`
        :return: view name or None if name can't be resolved
        :rtype: str or None
        """
        try:
            return resolve(request.path)._func_path
        except Resolver404:
            return None

    def process_view(self, request, *args, **kwargs):
        """Initialize statistics variables and environment.

        :return: Response object or None
        :rtype: :class:`django.http.HttpResponse` or None
        """
        if not profiler.is_on or self.match_exclude_urls(request.path):
            return

        # Force DB connection to debug mode to get sql time and number of queries
        self.force_debug_cursor = connection.force_debug_cursor
        connection.force_debug_cursor = True
        self.start_time = default_timer()

    def process_response(self, request, response):
        """Aggregates request and response statistics and saves it in profiler data.

        :param request: Request object
        :type request: :class:`django.http.HttpRequest`
        :param response: Response object returned by a Django view or by a middleware
        :type response: :class:`django.http.HttpResponse` or :class:`django.http.StreamingHttpResponse`
        :return: View response
        :rtype: :class:`django.http.HttpResponse` or :class:`django.http.StreamingHttpResponse`
        """
        if not profiler.is_on or self.match_exclude_urls(request.path) or not hasattr(request, 'user'):
            return response

        duration = default_timer() - self.start_time
        connection.force_debug_cursor = self.force_debug_cursor

        # Get SQL queries count and execution time
        sql_time = 0
        sql_count = len(connection.queries)
        for query in connection.queries:
            sql_time += float(query.get('time', 0))

        # Collects request and response params
        view_name = self.get_view_name(request)
        is_anon_call = request.user.is_anonymous() if callable(request.user.is_anonymous) else request.user.is_anonymous
        is_cache_hit = getattr(response, settings.SPEEDINFO_CACHED_RESPONSE_ATTR_NAME, False)

        # Saves profiler data
        if view_name is not None:
            profiler.process(view_name, request.method, is_anon_call, is_cache_hit, sql_time, sql_count, duration)

        return response

    def __call__(self, request):
        """New middleware handler introduced in Django 1.10.

        :param request: Request object
        :type request: :class:`django.http.HttpRequest`
        :return: View response
        :rtype: :class:`django.http.HttpResponse` or :class:`django.http.StreamingHttpResponse`
        """
        response = self.process_view(request)

        if response is None:
            response = self.get_response(request)

        return self.process_response(request, response)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-10-04 13:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ViewProfiler',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('view_name', models.CharField(max_length=255, verbose_name=b'View name')),
                ('method', models.CharField(max_length=8, verbose_name=b'HTTP method')),
                ('anon_calls', models.PositiveIntegerField(default=0, verbose_name=b'Anonymous calls')),
                ('cache_hits', models.PositiveIntegerField(default=0, verbose_name=b'Cache hits')),
                ('total_calls', models.PositiveIntegerField(default=0, verbose_name=b'Total calls')),
                ('total_time', models.FloatField(default=0, verbose_name=b'Total time')),
            ],
            options={
                'verbose_name_plural': 'Views profiler',
            },
        ),
    ]



# coding: utf-8

from django.conf.urls import url
from django.contrib import admin
from django.core.exceptions import PermissionDenied
from django.db.models import F, FloatField, ExpressionWrapper
from django.http import HttpResponseRedirect, HttpResponse

from speedinfo import profiler, settings
from speedinfo.models import ViewProfiler

try:
    from django.urls import reverse  # Django >= 1.10
except ImportError:
    from django.core.urlresolvers import reverse


class ViewProfilerAdmin(admin.ModelAdmin):
    list_display_links = None
    actions = None
    ordering = ('-total_time',)

    class Media:
        css = {
            'all': ('speedinfo/css/admin.css',)
        }

    def get_queryset(self, request):
        qs = super(ViewProfilerAdmin, self).get_queryset(request)
        return qs.annotate(
            percall=ExpressionWrapper(F('total_time') / F('total_calls'), output_field=FloatField())
        )

    def get_list_display(self, request):
        """Creates and returns list of callables to represent
        model field values with a custom format.

        :param request: Request object
        :type request: :class:`django.http.HttpRequest`
        :return: list containing fields to be displayed on the changelist
        """
        list_display = []

        for rc in settings.SPEEDINFO_REPORT_COLUMNS_FORMAT:
            if rc.attr_name in settings.SPEEDINFO_REPORT_COLUMNS:

                def wrapper(col):
                    def field_format(obj):
                        return col.format.format(getattr(obj, col.attr_name))
                    field_format.short_description = col.name
                    field_format.admin_order_field = col.order_field

                    return field_format

                list_display.append(
                    wrapper(rc)
                )

        return list_display

    def change_view(self, *args, **kwargs):
        raise PermissionDenied

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def changelist_view(self, request, extra_context=None):
        return super(ViewProfilerAdmin, self).changelist_view(request, extra_context={
            'title': 'Views profiler',
            'profiler_is_on': profiler.is_on,
        })

    def get_urls(self):
        return [
            url(r'^switch/$', self.admin_site.admin_view(self.switch), name='speedinfo-profiler-switch'),
            url(r'^export/$', self.admin_site.admin_view(self.export), name='speedinfo-profiler-export'),
            url(r'^reset/$', self.admin_site.admin_view(self.reset), name='speedinfo-profiler-reset'),
        ] + super(ViewProfilerAdmin, self).get_urls()

    def switch(self, request):
        profiler.switch()
        return HttpResponseRedirect(reverse('admin:speedinfo_viewprofiler_changelist'))

    def export(self, request):
        output = profiler.export()
        response = HttpResponse(output.getvalue(), content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename=profiler.csv'

        return response

    def reset(self, request):
        profiler.flush()
        return HttpResponseRedirect(reverse('admin:speedinfo_viewprofiler_changelist'))


admin.site.register(ViewProfiler, ViewProfilerAdmin)
